#!/usr/bin/env python
import os
import sys
import subprocess
import traceback
import time

from charmhelpers.core.host import set_nic_mtu
from charmhelpers.core.hookenv import (
    log, ERROR, config, status_set, relation_ids, related_units,
    relation_set, Hooks, UnregisteredHookError, unit_private_ip,
    local_unit)

from charmhelpers.contrib.python.packages import (pip_install)

dest_dir = os.path.join(os.environ.get('CHARM_DIR'), 'libs')
wheel_file = os.path.join(dest_dir, 'nexentaedge-1.0-py2-none-any.whl')
pip_install(wheel_file)

from nexentaedge.nedgeConfigurator import (NedgeNodeConfigurator)

from nexentaedge.utils import (get_sid)
from utils import (neadm_bundles_copy)

hooks = Hooks()


@hooks.hook('install')
def install():
    try:
        neadm_bundles_copy()
        log('Bundles has copied')
    except subprocess.CalledProcessError as cpe:
        log('Install failed!\nMessage:\n{0}\nTrace:\n{1}\nOutput:\n{2}'
            .format(cpe.message, traceback.format_exc(), cpe.output))
        sys.exit(1)
    except Exception as e:
        log('Installation failed!\nMessage:{0}\nTrace:{1}'
            .format(e.message, traceback.format_exc()))
        sys.exit(1)


@hooks.hook('config-changed')
def on_config_changed():

    if get_sid():
        log('Nexentaedge node already activated...')
        return

    replicast_eth = config('replicast_eth')
    private_ip = unit_private_ip()

    log('Node private ip is {}'.format(private_ip))
    environment = {
        'node_private_ip': private_ip,
        'replicast_eth': replicast_eth,
        'profile': config('profile'),
        'nodocker': config('nodocker'),
        'exclude': config('exclude'),
        'reserved': config('reserved')
    }

    #build 2240, should set replicast MTU to 9000
    replicast_eth = config('replicast_eth')
    if replicast_eth:
        set_nic_mtu(replicast_eth, '9000')

    status_set('maintenance', 'Installing nexentaedge...')
    configurator = NedgeNodeConfigurator(environment)
    if not configurator.configure():
        blockers = configurator.get_blockers()
        if blockers:
            status_set('blocked', blockers[0])
            sys.exet(0)
        log('NEDGE node configuration failed.', level=ERROR)
        sys.exit(1)

    time.sleep(60)

    #resend data to mgmt if relation already created
    for relid in relation_ids('nedge'):
        for unit in related_units(relid):
            log('resend data for relid:{0} unit:{1}'.format(relid, unit))
            on_join_to_mgmt(relid=relid, unit=unit)

    status_set('active', 'Ready')


@hooks.hook('nedge-relation-joined')
def on_join_to_mgmt(relid=None, unit=None):

    if not get_sid():
        return

    data = {
        'node_info': {
            'sid': get_sid(),
            'private_ip': unit_private_ip(),
            'unit_id': local_unit()
        }
    }

    relation_set(relation_id=relid,
                 relation_settings=data)
    log("[DATA] Send to MGMT {}".format(data))


if __name__ == '__main__':
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skipping.'.format(e))
